@@ -1,10 +1,10 @@
 /* chown-core.c -- core functions for changing ownership.
-   Copyright (C) 2000, 2002, 2003, 2004, 2005, 2006 Free Software Foundation.
+   Copyright (C) 2000, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Extracted from chown.c/chgrp.c and librarified by Jim Meyering.  */
 
@@ -32,6 +31,12 @@
 #include "root-dev-ino.h"
 #include "xfts.h"
 
+#define FTSENT_IS_DIRECTORY(E)	\
+  ((E)->fts_info == FTS_D	\
+   || (E)->fts_info == FTS_DC	\
+   || (E)->fts_info == FTS_DP	\
+   || (E)->fts_info == FTS_DNR)
+
 enum RCH_status
   {
     /* we called fchown and close, and both succeeded */
@@ -258,7 +263,20 @@ change_file_owner (FTS *fts, FTSENT *ent,
     {
     case FTS_D:
       if (chopt->recurse)
-	return true;
+	{
+	  if (ROOT_DEV_INO_CHECK (chopt->root_dev_ino, ent->fts_statp))
+	    {
+	      /* This happens e.g., with "chown -R --preserve-root 0 /"
+		 and with "chown -RH --preserve-root 0 symlink-to-root".  */
+	      ROOT_DEV_INO_WARN (file_full_name);
+	      /* Tell fts not to traverse into this hierarchy.  */
+	      fts_set (fts, ent, FTS_SKIP);
+	      /* Ensure that we do not process "/" on the second visit.  */
+	      ent = fts_read (fts);
+	      return false;
+	    }
+	  return true;
+	}
       break;
 
     case FTS_DP:
@@ -337,15 +355,13 @@ change_file_owner (FTS *fts, FTSENT *ent,
 		      || required_gid == file_stats->st_gid));
     }
 
-  if (do_chown
-      /* With FTS_NOSTAT, file_stats is valid only for directories.
-	 Don't need to check for FTS_D, since it is handled above,
-	 and same for FTS_DNR, since then do_chown is false.  */
-      && (ent->fts_info == FTS_DP || ent->fts_info == FTS_DC)
+  /* This happens when chown -LR --preserve-root encounters a symlink-to-/.  */
+  if (ok
+      && FTSENT_IS_DIRECTORY (ent)
       && ROOT_DEV_INO_CHECK (chopt->root_dev_ino, file_stats))
     {
       ROOT_DEV_INO_WARN (file_full_name);
-      ok = do_chown = false;
+      return false;
     }
 
   if (do_chown)
