@@ -1,10 +1,10 @@
 /* dd -- convert a file while copying it.
-   Copyright (C) 85, 90, 91, 1995-2006 Free Software Foundation, Inc.
+   Copyright (C) 85, 90, 91, 1995-2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Paul Rubin, David MacKenzie, and Stuart Kemp. */
 
@@ -29,7 +28,6 @@
 #include "error.h"
 #include "fd-reopen.h"
 #include "gethrxtime.h"
-#include "getpagesize.h"
 #include "human.h"
 #include "long-options.h"
 #include "quote.h"
@@ -265,7 +263,7 @@ static struct symbol_value const flags[] =
   {"direct",	O_DIRECT},
   {"directory",	O_DIRECTORY},
   {"dsync",	O_DSYNC},
-  {"noatime",	HAVE_WORKING_O_NOATIME ? O_NOATIME : 0},
+  {"noatime",	O_NOATIME},
   {"noctty",	O_NOCTTY},
   {"nofollow",	HAVE_WORKING_O_NOFOLLOW ? O_NOFOLLOW : 0},
   {"nolinks",	O_NOLINKS},
@@ -452,7 +450,7 @@ Each CONV symbol may be:\n\
       fputs (_("\
   noerror   continue after read errors\n\
   sync      pad every input block with NULs to ibs-size; when used\n\
-              with block or unblock, pad with spaces rather than NULs\n\
+            with block or unblock, pad with spaces rather than NULs\n\
   fdatasync  physically write output file data before finishing\n\
   fsync     likewise, but also write metadata\n\
 "), stdout);
@@ -472,7 +470,7 @@ Each FLAG symbol may be:\n\
 	fputs (_("  sync      likewise, but also for metadata\n"), stdout);
       if (O_NONBLOCK)
 	fputs (_("  nonblock  use non-blocking I/O\n"), stdout);
-      if (HAVE_WORKING_O_NOATIME)
+      if (O_NOATIME)
 	fputs (_("  noatime   do not update access time\n"), stdout);
       if (O_NOCTTY)
 	fputs (_("  noctty    do not assign controlling terminal from file\n"),
@@ -507,7 +505,7 @@ Options are:\n\
 
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -873,6 +871,7 @@ static void
 scanargs (int argc, char **argv)
 {
   int i;
+  size_t blocksize = 0;
 
   for (i = optind; i < argc; i++)
     {
@@ -923,7 +922,7 @@ scanargs (int argc, char **argv)
 	  else if (STREQ (name, "bs"))
 	    {
 	      invalid |= ! (0 < n && n <= MAX_BLOCKSIZE (INPUT_BLOCK_SLOP));
-	      output_blocksize = input_blocksize = n;
+	      blocksize = n;
 	    }
 	  else if (STREQ (name, "cbs"))
 	    {
@@ -948,6 +947,9 @@ scanargs (int argc, char **argv)
 	}
     }
 
+  if (blocksize)
+    input_blocksize = output_blocksize = blocksize;
+
   /* If bs= was given, both `input_blocksize' and `output_blocksize' will
      have been set to positive values.  If either has not been set,
      bs= was not given, so make sure two buffers are used. */
@@ -1222,7 +1224,7 @@ advance_input_after_read_error (size_t nbytes)
 	    return true;
 	  diff = input_offset - offset;
 	  if (! (0 <= diff && diff <= nbytes))
-	    error (0, 0, _("warning: screwy file offset after failed read"));
+	    error (0, 0, _("warning: invalid file offset after failed read"));
 	  if (0 <= skip_via_lseek (input_file, STDIN_FILENO, diff, SEEK_CUR))
 	    return true;
 	  if (errno == 0)
