@@ -1,11 +1,11 @@
 /* dircolors - output commands to set the LS_COLOR environment variable
-   Copyright (C) 1996-2006 Free Software Foundation, Inc.
+   Copyright (C) 1996-2007 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995, 1997, 1998, 1999, 2000 H. Peter Anvin
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -24,11 +23,10 @@
 
 #include "system.h"
 #include "dircolors.h"
+#include "c-strcase.h"
 #include "error.h"
-#include "getline.h"
 #include "obstack.h"
 #include "quote.h"
-#include "strcase.h"
 #include "xstrndup.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
@@ -116,7 +114,7 @@ If FILE is specified, read it to determine which colors to use for which\n\
 file types and extensions.  Otherwise, a precompiled database is used.\n\
 For details on the format of these files, run `dircolors --print-database'.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
 
   exit (status);
@@ -246,7 +244,7 @@ dc_parse_stream (FILE *fp, const char *filename)
   char *input_line = NULL;
   size_t input_line_size = 0;
   char const *line;
-  char *term;
+  char const *term;
   bool ok = true;
 
   /* State for the parser.  */
@@ -296,7 +294,7 @@ dc_parse_stream (FILE *fp, const char *filename)
 	}
 
       unrecognized = false;
-      if (strcasecmp (keywd, "TERM") == 0)
+      if (c_strcasecmp (keywd, "TERM") == 0)
 	{
 	  if (STREQ (arg, term))
 	    state = ST_TERMSURE;
@@ -325,9 +323,9 @@ dc_parse_stream (FILE *fp, const char *filename)
 		  append_quoted (arg);
 		  APPEND_CHAR (':');
 		}
-	      else if (strcasecmp (keywd, "OPTIONS") == 0
-		       || strcasecmp (keywd, "COLOR") == 0
-		       || strcasecmp (keywd, "EIGHTBIT") == 0)
+	      else if (c_strcasecmp (keywd, "OPTIONS") == 0
+		       || c_strcasecmp (keywd, "COLOR") == 0
+		       || c_strcasecmp (keywd, "EIGHTBIT") == 0)
 		{
 		  /* Ignore.  */
 		}
@@ -336,7 +334,7 @@ dc_parse_stream (FILE *fp, const char *filename)
 		  int i;
 
 		  for (i = 0; slack_codes[i] != NULL; ++i)
-		    if (strcasecmp (keywd, slack_codes[i]) == 0)
+		    if (c_strcasecmp (keywd, slack_codes[i]) == 0)
 		      break;
 
 		  if (slack_codes[i] != NULL)
