@@ -2,7 +2,7 @@ static char const G_line[] =
 {
   '#',' ','C','o','n','f','i','g','u','r','a','t','i','o','n',' ','f','i','l','e',' ','f','o','r',' ','d','i','r','c','o','l','o','r','s',',',' ','a',' ','u','t','i','l','i','t','y',' ','t','o',' ','h','e','l','p',' ','y','o','u',' ','s','e','t',' ','t','h','e',0,
   '#',' ','L','S','_','C','O','L','O','R','S',' ','e','n','v','i','r','o','n','m','e','n','t',' ','v','a','r','i','a','b','l','e',' ','u','s','e','d',' ','b','y',' ','G','N','U',' ','l','s',' ','w','i','t','h',' ','t','h','e',' ','-','-','c','o','l','o','r',' ','o','p','t','i','o','n','.',0,
-  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6',',',' ','1','9','9','9',',',' ','2','0','0','0',',',' ','2','0','0','1',',',' ','2','0','0','2',',',' ','2','0','0','3',',',' ','2','0','0','4',',',' ','2','0','0','5',',',' ','2','0','0','6',0,
+  '#',' ','C','o','p','y','r','i','g','h','t',' ','(','C',')',' ','1','9','9','6',',',' ','1','9','9','9','-','2','0','0','8',0,
   '#',' ','F','r','e','e',' ','S','o','f','t','w','a','r','e',' ','F','o','u','n','d','a','t','i','o','n',',',' ','I','n','c','.',0,
   '#',' ','C','o','p','y','i','n','g',' ','a','n','d',' ','d','i','s','t','r','i','b','u','t','i','o','n',' ','o','f',' ','t','h','i','s',' ','f','i','l','e',',',' ','w','i','t','h',' ','o','r',' ','w','i','t','h','o','u','t',' ','m','o','d','i','f','i','c','a','t','i','o','n',',',0,
   '#',' ','a','r','e',' ','p','e','r','m','i','t','t','e','d',' ','p','r','o','v','i','d','e','d',' ','t','h','e',' ','c','o','p','y','r','i','g','h','t',' ','n','o','t','i','c','e',' ','a','n','d',' ','t','h','i','s',' ','n','o','t','i','c','e',' ','a','r','e',' ','p','r','e','s','e','r','v','e','d','.',0,
@@ -22,10 +22,13 @@ static char const G_line[] =
   'T','E','R','M',' ','c','o','n','8','0','x','4','3',0,
   'T','E','R','M',' ','c','o','n','8','0','x','5','0',0,
   'T','E','R','M',' ','c','o','n','8','0','x','6','0',0,
+  'T','E','R','M',' ','c','o','n','s','2','5',0,
   'T','E','R','M',' ','c','o','n','s','o','l','e',0,
   'T','E','R','M',' ','c','y','g','w','i','n',0,
   'T','E','R','M',' ','d','t','t','e','r','m',0,
+  'T','E','R','M',' ','e','t','e','r','m','-','c','o','l','o','r',0,
   'T','E','R','M',' ','g','n','o','m','e',0,
+  'T','E','R','M',' ','g','n','o','m','e','-','2','5','6','c','o','l','o','r',0,
   'T','E','R','M',' ','k','o','n','s','o','l','e',0,
   'T','E','R','M',' ','k','t','e','r','m',0,
   'T','E','R','M',' ','l','i','n','u','x',0,
@@ -38,12 +41,15 @@ static char const G_line[] =
   'T','E','R','M',' ','r','x','v','t','-','c','y','g','w','i','n','-','n','a','t','i','v','e',0,
   'T','E','R','M',' ','r','x','v','t','-','u','n','i','c','o','d','e',0,
   'T','E','R','M',' ','s','c','r','e','e','n',0,
+  'T','E','R','M',' ','s','c','r','e','e','n','-','2','5','6','c','o','l','o','r',0,
   'T','E','R','M',' ','s','c','r','e','e','n','-','b','c','e',0,
   'T','E','R','M',' ','s','c','r','e','e','n','-','w',0,
   'T','E','R','M',' ','s','c','r','e','e','n','.','l','i','n','u','x',0,
   'T','E','R','M',' ','v','t','1','0','0',0,
   'T','E','R','M',' ','x','t','e','r','m',0,
+  'T','E','R','M',' ','x','t','e','r','m','-','1','6','c','o','l','o','r',0,
   'T','E','R','M',' ','x','t','e','r','m','-','2','5','6','c','o','l','o','r',0,
+  'T','E','R','M',' ','x','t','e','r','m','-','8','8','c','o','l','o','r',0,
   'T','E','R','M',' ','x','t','e','r','m','-','c','o','l','o','r',0,
   'T','E','R','M',' ','x','t','e','r','m','-','d','e','b','i','a','n',0,
   '#',' ','B','e','l','o','w',' ','a','r','e',' ','t','h','e',' ','c','o','l','o','r',' ','i','n','i','t',' ','s','t','r','i','n','g','s',' ','f','o','r',' ','t','h','e',' ','b','a','s','i','c',' ','f','i','l','e',' ','t','y','p','e','s','.',' ','A',' ','c','o','l','o','r',' ','i','n','i','t',0,
@@ -88,12 +94,15 @@ static char const G_line[] =
   ' ','#',' ','a','r','c','h','i','v','e','s',' ','o','r',' ','c','o','m','p','r','e','s','s','e','d',' ','(','b','r','i','g','h','t',' ','r','e','d',')',0,
   '.','t','a','r',' ','0','1',';','3','1',0,
   '.','t','g','z',' ','0','1',';','3','1',0,
+  '.','s','v','g','z',' ','0','1',';','3','1',0,
   '.','a','r','j',' ','0','1',';','3','1',0,
   '.','t','a','z',' ','0','1',';','3','1',0,
   '.','l','z','h',' ','0','1',';','3','1',0,
+  '.','l','z','m','a',' ','0','1',';','3','1',0,
   '.','z','i','p',' ','0','1',';','3','1',0,
   '.','z',' ','0','1',';','3','1',0,
   '.','Z',' ','0','1',';','3','1',0,
+  '.','d','z',' ','0','1',';','3','1',0,
   '.','g','z',' ','0','1',';','3','1',0,
   '.','b','z','2',' ','0','1',';','3','1',0,
   '.','b','z',' ','0','1',';','3','1',0,
@@ -122,6 +131,7 @@ static char const G_line[] =
   '.','t','i','f',' ','0','1',';','3','5',0,
   '.','t','i','f','f',' ','0','1',';','3','5',0,
   '.','p','n','g',' ','0','1',';','3','5',0,
+  '.','s','v','g',' ','0','1',';','3','5',0,
   '.','m','n','g',' ','0','1',';','3','5',0,
   '.','p','c','x',' ','0','1',';','3','5',0,
   '.','m','o','v',' ','0','1',';','3','5',0,
