@@ -1,10 +1,10 @@
 /* nl -- number lines of files
-   Copyright (C) 89, 92, 1995-2004 Free Software Foundation, Inc.
+   Copyright (C) 89, 92, 1995-2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Scott Bartram (nancy!scott@uunet.uu.net)
    Revised by David MacKenzie (djm@gnu.ai.mit.edu) */
@@ -64,16 +63,16 @@ enum section
 char *program_name;
 
 /* Format of body lines (-b).  */
-static char *body_type = "t";
+static char const *body_type = "t";
 
 /* Format of header lines (-h).  */
-static char *header_type = "n";
+static char const *header_type = "n";
 
 /* Format of footer lines (-f).  */
-static char *footer_type = "n";
+static char const *footer_type = "n";
 
 /* Format currently being used (body, header, or footer).  */
-static char *current_type;
+static char const *current_type;
 
 /* Regex for body lines to number (-bp).  */
 static struct re_pattern_buffer body_regex;
@@ -84,11 +83,16 @@ static struct re_pattern_buffer header_regex;
 /* Regex for footer lines to number (-fp).  */
 static struct re_pattern_buffer footer_regex;
 
+/* Fastmaps for the above.  */
+static char body_fastmap[UCHAR_MAX + 1];
+static char header_fastmap[UCHAR_MAX + 1];
+static char footer_fastmap[UCHAR_MAX + 1];
+
 /* Pointer to current regex, if any.  */
 static struct re_pattern_buffer *current_regex = NULL;
 
 /* Separator string to print after line number (-s).  */
-static char *separator_str = "\t";
+static char const *separator_str = "\t";
 
 /* Input section delimiter string (-d).  */
 static char const *section_del = DEFAULT_SECTION_DELIMITERS;
@@ -212,7 +216,7 @@ second character implies :.  Type \\\\ for \\.  STYLE is one of:\n\
   t         number only nonempty lines\n\
   n         number no lines\n\
   pBRE      number only lines that contain a match for the basic regular\n\
-              expression, BRE\n\
+            expression, BRE\n\
 \n\
 FORMAT is one of:\n\
 \n\
@@ -221,7 +225,7 @@ FORMAT is one of:\n\
   rz   right justified, leading zeros\n\
 \n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -230,11 +234,11 @@ FORMAT is one of:\n\
    according to `optarg'.  */
 
 static bool
-build_type_arg (char **typep, struct re_pattern_buffer *regexp)
+build_type_arg (char const **typep,
+		struct re_pattern_buffer *regexp, char *fastmap)
 {
-  const char *errmsg;
+  char const *errmsg;
   bool rval = true;
-  size_t optlen;
 
   switch (*optarg)
     {
@@ -245,13 +249,13 @@ build_type_arg (char **typep, struct re_pattern_buffer *regexp)
       break;
     case 'p':
       *typep = optarg++;
-      optlen = strlen (optarg);
-      regexp->allocated = optlen * 2;
-      regexp->buffer = xmalloc (regexp->allocated);
+      regexp->buffer = NULL;
+      regexp->allocated = 0;
+      regexp->fastmap = fastmap;
       regexp->translate = NULL;
-      regexp->fastmap = xmalloc (256);
-      regexp->fastmap_accurate = 0;
-      errmsg = re_compile_pattern (optarg, optlen, regexp);
+      re_syntax_options =
+	RE_SYNTAX_POSIX_BASIC & ~RE_CONTEXT_INVALID_DUP & ~RE_NO_EMPTY_RANGES;
+      errmsg = re_compile_pattern (optarg, strlen (optarg), regexp);
       if (errmsg)
 	error (EXIT_FAILURE, 0, "%s", errmsg);
       break;
@@ -342,12 +346,20 @@ proc_text (void)
       fputs (print_no_line_fmt, stdout);
       break;
     case 'p':
-      if (re_search (current_regex, line_buf.buffer, line_buf.length - 1,
-		     0, line_buf.length - 1, (struct re_registers *) 0) < 0)
-	fputs (print_no_line_fmt, stdout);
-      else
-	print_lineno ();
-      break;
+      switch (re_search (current_regex, line_buf.buffer, line_buf.length - 1,
+			 0, line_buf.length - 1, NULL))
+	{
+	case -2:
+	  error (EXIT_FAILURE, errno, _("error in regular expression search"));
+
+	case -1:
+	  fputs (print_no_line_fmt, stdout);
+	  break;
+
+	default:
+	  print_lineno ();
+	  break;
+	}
     }
   fwrite (line_buf.buffer, sizeof (char), line_buf.length, stdout);
 }
@@ -402,7 +414,7 @@ process_file (FILE *fp)
    Return true if successful.  */
 
 static bool
-nl_file (const char *file)
+nl_file (char const *file)
 {
   FILE *stream;
 
@@ -461,7 +473,7 @@ main (int argc, char **argv)
       switch (c)
 	{
 	case 'h':
-	  if (! build_type_arg (&header_type, &header_regex))
+	  if (! build_type_arg (&header_type, &header_regex, header_fastmap))
 	    {
 	      error (0, 0, _("invalid header numbering style: %s"),
 		     quote (optarg));
@@ -469,7 +481,7 @@ main (int argc, char **argv)
 	    }
 	  break;
 	case 'b':
-	  if (! build_type_arg (&body_type, &body_regex))
+	  if (! build_type_arg (&body_type, &body_regex, body_fastmap))
 	    {
 	      error (0, 0, _("invalid body numbering style: %s"),
 		     quote (optarg));
@@ -477,7 +489,7 @@ main (int argc, char **argv)
 	    }
 	  break;
 	case 'f':
-	  if (! build_type_arg (&footer_type, &footer_regex))
+	  if (! build_type_arg (&footer_type, &footer_regex, footer_fastmap))
 	    {
 	      error (0, 0, _("invalid footer numbering style: %s"),
 		     quote (optarg));
