@@ -1,10 +1,10 @@
 /* pr -- convert text files for printing.
    Copyright (C) 88, 91, 1995-2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /*  By Pete TerMaat, with considerable refinement by Roland Huebner.  */
 
@@ -313,32 +312,6 @@
 
 #include <getopt.h>
 #include <sys/types.h>
-
-/* Get MB_LEN_MAX.  */
-#include <limits.h>
-/* MB_LEN_MAX is incorrectly defined to be 1 in at least one GCC
-   installation; work around this configuration error.  */
-#if !defined MB_LEN_MAX || MB_LEN_MAX == 1
-# define MB_LEN_MAX 16
-#endif
-
-/* Get MB_CUR_MAX.  */
-#include <stdlib.h>
-
-/* Solaris 2.5 has a bug: <wchar.h> must be included before <wctype.h>.  */
-/* Get mbstate_t, mbrtowc(), wcwidth().  */
-#if HAVE_WCHAR_H
-# include <wchar.h>
-#endif
-
-/* Get iswprint(). -- for wcwidth().  */
-#if HAVE_WCTYPE_H
-# include <wctype.h>
-#endif
-#if !defined iswprint && !HAVE_ISWPRINT
-# define iswprint(wc) 1
-#endif
-
 #include "system.h"
 #include "error.h"
 #include "hard-locale.h"
@@ -350,18 +323,6 @@
 #include "strftime.h"
 #include "xstrtol.h"
 
-/* Some systems, like BeOS, have multibyte encodings but lack mbstate_t.  */
-#if HAVE_MBRTOWC && defined mbstate_t
-# define mbrtowc(pwc, s, n, ps) (mbrtowc) (pwc, s, n, 0)
-#endif
-
-#ifndef HAVE_DECL_WCWIDTH
-"this configure-time declaration test was not run"
-#endif
-#if !HAVE_DECL_WCWIDTH
-extern int wcwidth ();
-#endif
-
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "pr"
 
@@ -454,20 +415,7 @@ typedef struct COLUMN COLUMN;
 
 #define NULLCOL (COLUMN *)0
 
-/* Funtion pointers to switch functions for single byte locale or for
-   multibyte locale. If multibyte functions do not exist in your sysytem,
-   these pointers always point the function for single byte locale. */
-static void (*print_char) (char c);
-static int (*char_to_clump) (char c);
-
-/* Functions for single byte locale. */
-static void print_char_single (char c);
-static int char_to_clump_single (char c);
-
-/* Functions for multibyte locale. */
-static void print_char_multi (char c);
-static int char_to_clump_multi (char c);
-
+static int char_to_clump (char c);
 static bool read_line (COLUMN *p);
 static bool print_page (void);
 static bool print_stored (COLUMN *p);
@@ -477,7 +425,6 @@ static void print_header (void);
 static void pad_across_to (int position);
 static void add_line_number (COLUMN *p);
 static void getoptarg (char *arg, char switch_char, char *character,
-		       int *character_length, int *character_width,
 		       int *number);
 void usage (int status);
 static void print_files (int number_of_files, char **av);
@@ -492,6 +439,7 @@ static void store_char (char c);
 static void pad_down (int lines);
 static void read_rest_of_line (COLUMN *p);
 static void skip_read (COLUMN *p, int column_number);
+static void print_char (char c);
 static void cleanup (void);
 static void print_sep_string (void);
 static void separator_string (const char *optarg_S);
@@ -506,7 +454,7 @@ static COLUMN *column_vector;
    we store the leftmost columns contiguously in buff.
    To print a line from buff, get the index of the first character
    from line_vector[i], and print up to line_vector[i + 1]. */
-static unsigned char *buff;
+static char *buff;
 
 /* Index of the position in buff where the next character
    will be stored. */
@@ -587,9 +535,9 @@ static int lines_per_page = 66;
 
 /* Number of lines in the header and footer can be reset to 0 using
    the -t flag. */
-static int lines_per_header = 5;
+enum { lines_per_header = 5 };
 static int lines_per_body;
-static int lines_per_footer = 5;
+enum { lines_per_footer = 5 };
 
 /* (-w|-W) Width in characters of the page.  Does not include the width of
    the margin. */
@@ -610,7 +558,7 @@ static int chars_per_column;
 static bool untabify_input = false;
 
 /* (-e) The input tab character. */
-static char input_tab_char[MB_LEN_MAX] = "\t";
+static char input_tab_char = '\t';
 
 /* (-e) Tabstops are at chars_per_tab, 2*chars_per_tab, 3*chars_per_tab, ...
    where the leftmost column is 1. */
@@ -620,10 +568,7 @@ static int chars_per_input_tab = 8;
 static bool tabify_output = false;
 
 /* (-i) The output tab character. */
-static char output_tab_char[MB_LEN_MAX] = "\t";
-
-/* (-i) The byte length of output tab character. */
-static int output_tab_char_length = 1;
+static char output_tab_char = '\t';
 
 /* (-i) The width of the output tab. */
 static int chars_per_output_tab = 8;
@@ -697,13 +642,7 @@ static int power_10;
 static bool numbered_lines = false;
 
 /* (-n) Character which follows each line number. */
-static char number_separator[MB_LEN_MAX] = "\t";
-
-/* (-n) The byte length of the character which follows each line number. */
-static int number_separator_length = 1;
-
-/* (-n) The character width of the character which follows each line number. */
-static int number_separator_width = 0;
+static char number_separator = '\t';
 
 /* (-n) line counting starts with 1st line of input file (not with 1st
    line of 1st page printed). */
@@ -756,7 +695,6 @@ static bool use_col_separator = false;
    -a|COLUMN|-m is a `space' and with the -J option a `tab'. */
 static char *col_sep_string = "";
 static int col_sep_length = 0;
-static int col_sep_width = 0;
 static char *column_separator = " ";
 static char *line_separator = "\t";
 
@@ -858,14 +796,14 @@ cols_ready_to_print (void)
    using option +FIRST_PAGE:LAST_PAGE */
 
 static bool
-first_last_page (char const *pages)
+first_last_page (int oi, char c, char const *pages)
 {
   char *p;
   uintmax_t first;
   uintmax_t last = UINTMAX_MAX;
   strtol_error err = xstrtoumax (pages, &p, 10, &first, "");
   if (err != LONGINT_OK && err != LONGINT_INVALID_SUFFIX_CHAR)
-    _STRTOL_ERROR (EXIT_FAILURE, pages, _("page range"), err);
+    xstrtol_fatal (err, oi, c, long_options, pages);
 
   if (p == pages || !first)
     return false;
@@ -875,7 +813,7 @@ first_last_page (char const *pages)
       char const *p1 = p + 1;
       err = xstrtoumax (p1, &p, 10, &last, "");
       if (err != LONGINT_OK)
-	_STRTOL_ERROR (EXIT_FAILURE, pages, _("page range"), err);
+	xstrtol_fatal (err, oi, c, long_options, pages);
       if (p1 == p || last < first)
 	return false;
     }
@@ -913,19 +851,11 @@ separator_string (const char *optarg_S)
   col_sep_length = (int) strlen (optarg_S);
   col_sep_string = xmalloc (col_sep_length + 1);
   strcpy (col_sep_string, optarg_S);
-
-#if HAVE_MBRTOWC
-  if (MB_CUR_MAX > 1)
-    col_sep_width = mbswidth (col_sep_string, 0);
-  else
-#endif
-    col_sep_width = col_sep_length;
 }
 
 int
 main (int argc, char **argv)
 {
-  int c;
   int n_files;
   bool old_options = false;
   bool old_w = false;
@@ -945,29 +875,18 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-/* Define which functions are used, the ones for single byte locale or the ones
-   for multibyte locale. */
-#if HAVE_MBRTOWC
-  if (MB_CUR_MAX > 1)
-    {
-      print_char = print_char_multi;
-      char_to_clump = char_to_clump_multi;
-    }
-  else
-#endif
-    {
-      print_char = print_char_single;
-      char_to_clump = char_to_clump_single;
-    }
-
   n_files = 0;
   file_names = (argc > 1
 		? xmalloc ((argc - 1) * sizeof (char *))
 		: NULL);
 
-  while ((c = getopt_long (argc, argv, short_options, long_options, NULL))
-	 != -1)
+  for (;;)
     {
+      int oi = -1;
+      int c = getopt_long (argc, argv, short_options, long_options, &oi);
+      if (c == -1)
+	break;
+
       if (ISDIGIT (c))
 	{
 	  /* Accumulate column-count digits specified via old-style options. */
@@ -986,7 +905,7 @@ main (int argc, char **argv)
 	case 1:			/* Non-option argument. */
 	  /* long option --page dominates old `+FIRST_PAGE ...'.  */
 	  if (! (first_page_number == 0
-		 && *optarg == '+' && first_last_page (optarg + 1)))
+		 && *optarg == '+' && first_last_page (-2, '+', optarg + 1)))
 	    file_names[n_files++] = optarg;
 	  break;
 
@@ -995,7 +914,7 @@ main (int argc, char **argv)
 	    if (! optarg)
 	      error (EXIT_FAILURE, 0,
 		     _("`--pages=FIRST_PAGE[:LAST_PAGE]' missing argument"));
-	    else if (! first_last_page (optarg))
+	    else if (! first_last_page (oi, 0, optarg))
 	      error (EXIT_FAILURE, 0, _("Invalid page range %s"),
 		     quote (optarg));
 	    break;
@@ -1032,12 +951,8 @@ main (int argc, char **argv)
 	  break;
 	case 'e':
 	  if (optarg)
-	    {
-	      int dummy_length, dummy_width;
-
-	      getoptarg (optarg, 'e', input_tab_char, &dummy_length,
-			 &dummy_width, &chars_per_input_tab);
-	    }
+	    getoptarg (optarg, 'e', &input_tab_char,
+		       &chars_per_input_tab);
 	  /* Could check tab width > 0. */
 	  untabify_input = true;
 	  break;
@@ -1050,12 +965,8 @@ main (int argc, char **argv)
 	  break;
 	case 'i':
 	  if (optarg)
-	    {
-	      int dummy_width;
-
-	      getoptarg (optarg, 'i', output_tab_char, &output_tab_char_length,
-			 &dummy_width, &chars_per_output_tab);
-	    }
+	    getoptarg (optarg, 'i', &output_tab_char,
+		       &chars_per_output_tab);
 	  /* Could check tab width > 0. */
 	  tabify_output = true;
 	  break;
@@ -1082,8 +993,8 @@ main (int argc, char **argv)
 	case 'n':
 	  numbered_lines = true;
 	  if (optarg)
-	    getoptarg (optarg, 'n', number_separator, &number_separator_length,
-		       &number_separator_width, &chars_per_number);
+	    getoptarg (optarg, 'n', &number_separator,
+		       &chars_per_number);
 	  break;
 	case 'N':
 	  skip_count = false;
@@ -1122,7 +1033,7 @@ main (int argc, char **argv)
 	  old_s = false;
 	  /* Reset an additional input of -s, -S dominates -s */
 	  col_sep_string = "";
-	  col_sep_length = col_sep_width = 0;
+	  col_sep_length = 0;
 	  use_col_separator = true;
 	  if (optarg)
 	    separator_string (optarg);
@@ -1279,45 +1190,10 @@ main (int argc, char **argv)
    a number. */
 
 static void
-getoptarg (char *arg, char switch_char, char *character, int *character_length,
-	   int *character_width, int *number)
+getoptarg (char *arg, char switch_char, char *character, int *number)
 {
   if (!ISDIGIT (*arg))
-    {
-#ifdef HAVE_MBRTOWC
-      if (MB_CUR_MAX > 1)	/* for multibyte locale. */
-	{
-	  wchar_t wc;
-	  size_t mblength;
-	  int width;
-	  mbstate_t state = {'\0'};
-
-	  mblength = mbrtowc (&wc, arg, strnlen(arg, MB_LEN_MAX), &state);
-
-	  if (mblength == (size_t)-1 || mblength == (size_t)-2)
-	    {
-	      *character_length = 1;
-	      *character_width = 1;
-	    }
-	  else
-	    {
-	      *character_length = (mblength < 1) ? 1 : mblength;
-	      width = wcwidth (wc);
-	      *character_width = (width < 0) ? 0 : width;
-	    }
-
-	  strncpy (character, arg, *character_length);
-	  arg += *character_length;
-	}
-      else			/* for single byte locale. */
-#endif
-	{
-	  *character = *arg++;
-	  *character_length = 1;
-	  *character_width = 1;
-	}
-    }
-
+    *character = *arg++;
   if (*arg)
     {
       long int tmp_long;
@@ -1340,12 +1216,6 @@ init_parameters (int number_of_files)
 {
   int chars_used_by_number = 0;
 
-  if (use_form_feed)
-    {
-      lines_per_header = 3;
-      lines_per_footer = 0;
-    }
-
   lines_per_body = lines_per_page - lines_per_header - lines_per_footer;
   if (lines_per_body <= 0)
     {
@@ -1382,7 +1252,7 @@ init_parameters (int number_of_files)
 	  else
 	    col_sep_string = column_separator;
 
-	  col_sep_length = col_sep_width = 1;
+	  col_sep_length = 1;
 	  use_col_separator = true;
 	}
       /* It's rather pointless to define a TAB separator with column
@@ -1413,11 +1283,11 @@ init_parameters (int number_of_files)
 	     TAB_WIDTH (chars_per_input_tab, chars_per_number);   */
 
       /* Estimate chars_per_text without any margin and keep it constant. */
-      if (number_separator[0] == '\t')
+      if (number_separator == '\t')
 	number_width = chars_per_number +
 	  TAB_WIDTH (chars_per_default_tab, chars_per_number);
       else
-	number_width = chars_per_number + number_separator_width;
+	number_width = chars_per_number + 1;
 
       /* The number is part of the column width unless we are
 	 printing files in parallel. */
@@ -1432,7 +1302,7 @@ init_parameters (int number_of_files)
     }
 
   chars_per_column = (chars_per_line - chars_used_by_number -
-		     (columns - 1) * col_sep_width) / columns;
+		     (columns - 1) * col_sep_length) / columns;
 
   if (chars_per_column < 1)
     error (EXIT_FAILURE, 0, _("page width too narrow"));
@@ -1557,7 +1427,7 @@ init_funcs (void)
 
   /* Enlarge p->start_position of first column to use the same form of
      padding_not_printed with all columns. */
-  h = h + col_sep_width;
+  h = h + col_sep_length;
 
   /* This loop takes care of all but the rightmost column. */
 
@@ -1591,7 +1461,7 @@ init_funcs (void)
 	}
       else
 	{
-	  h = h_next + col_sep_width;
+	  h = h_next + col_sep_length;
 	  h_next = h + chars_per_column;
 	}
     }
@@ -1881,9 +1751,9 @@ static void
 align_column (COLUMN *p)
 {
   padding_not_printed = p->start_position;
-  if (padding_not_printed - col_sep_width > 0)
+  if (padding_not_printed - col_sep_length > 0)
     {
-      pad_across_to (padding_not_printed - col_sep_width);
+      pad_across_to (padding_not_printed - col_sep_length);
       padding_not_printed = ANYWHERE;
     }
 
@@ -2154,13 +2024,13 @@ store_char (char c)
       /* May be too generous. */
       buff = X2REALLOC (buff, &buff_allocated);
     }
-  buff[buff_current++] = (unsigned char) c;
+  buff[buff_current++] = c;
 }
 
 static void
 add_line_number (COLUMN *p)
 {
-  int i, j;
+  int i;
   char *s;
   int left_cut;
 
@@ -2183,24 +2053,22 @@ add_line_number (COLUMN *p)
       /* Tabification is assumed for multiple columns, also for n-separators,
 	 but `default n-separator = TAB' hasn't been given priority over
 	 equal column_width also specified by POSIX. */
-      if (number_separator[0] == '\t')
+      if (number_separator == '\t')
         {
           i = number_width - chars_per_number;
           while (i-- > 0)
 	    (p->char_func) (' ');
         }
       else
-	for (j = 0; j < number_separator_length; j++)
-	  (p->char_func) (number_separator[j]);
+        (p->char_func) (number_separator);
     }
   else
     /* To comply with POSIX, we avoid any expansion of default TAB
        separator with a single column output. No column_width requirement
        has to be considered. */
     {
-      for (j = 0; j < number_separator_length; j++)
-	(p->char_func) (number_separator[j]);
-      if (number_separator[0] == '\t')
+      (p->char_func) (number_separator);
+      if (number_separator == '\t')
         output_position = POS_AFTER_TAB (chars_per_output_tab,
 			  output_position);
     }
@@ -2361,7 +2229,7 @@ print_white_space (void)
   while (goal - h_old > 1
 	 && (h_new = POS_AFTER_TAB (chars_per_output_tab, h_old)) <= goal)
     {
-      fwrite (output_tab_char, sizeof(char), output_tab_char_length, stdout);
+      putchar (output_tab_char);
       h_old = h_new;
     }
   while (++h_old <= goal)
@@ -2381,7 +2249,6 @@ print_sep_string (void)
 {
   char *s;
   int l = col_sep_length;
-  int not_space_flag;
 
   s = col_sep_string;
 
@@ -2395,7 +2262,6 @@ print_sep_string (void)
     {
       for (; separators_not_printed > 0; --separators_not_printed)
 	{
-	  not_space_flag = 0;
 	  while (l-- > 0)
 	    {
 	      /* 3 types of sep_strings: spaces only, spaces and chars,
@@ -2409,15 +2275,12 @@ print_sep_string (void)
 		}
 	      else
 		{
-		  not_space_flag = 1;
 		  if (spaces_not_printed > 0)
 		    print_white_space ();
 		  putchar (*s++);
+		  ++output_position;
 		}
 	    }
-	  if (not_space_flag)
-	    output_position += col_sep_width;
-
           /* sep_string ends with some spaces */
 	  if (spaces_not_printed > 0)
 	    print_white_space ();
@@ -2445,7 +2308,7 @@ print_clump (COLUMN *p, int n, char *clump)
    required number of tabs and spaces. */
 
 static void
-print_char_single (char c)
+print_char (char c)
 {
   if (tabify_output)
     {
@@ -2469,74 +2332,6 @@ print_char_single (char c)
   putchar (c);
 }
 
-#ifdef HAVE_MBRTOWC
-static void
-print_char_multi (char c)
-{
-  static size_t mbc_pos = 0;
-  static char mbc[MB_LEN_MAX] = {'\0'};
-  static mbstate_t state = {'\0'};
-  mbstate_t state_bak;
-  wchar_t wc;
-  size_t mblength;
-  int width;
-
-  if (tabify_output)
-    {
-      state_bak = state;
-      mbc[mbc_pos++] = c;
-      mblength = mbrtowc (&wc, mbc, mbc_pos, &state);
-
-      while (mbc_pos > 0)
-	{
-	  switch (mblength)
-	    {
-	    case (size_t)-2:
-	      state = state_bak;
-	      return;
-
-	    case (size_t)-1:
-	      state = state_bak;
-	      ++output_position;
-	      putchar (mbc[0]);
-	      memmove (mbc, mbc + 1, MB_CUR_MAX - 1);
-	      --mbc_pos;
-	      break;
-
-	    case 0:
-	      mblength = 1;
-
-	    default:
-	      if (wc == L' ')
-		{
-		  memmove (mbc, mbc + mblength, MB_CUR_MAX - mblength);
-		  --mbc_pos;
-		  ++spaces_not_printed;
-		  return;
-		}
-	      else if (spaces_not_printed > 0)
-		print_white_space ();
-
-	      /* Nonprintables are assumed to have width 0, except L'\b'. */
-	      if ((width = wcwidth (wc)) < 1)
-		{
-		  if (wc == L'\b')
-		    --output_position;
-		}
-	      else
-		output_position += width;
-
-	      fwrite (mbc, sizeof(char), mblength, stdout);
-	      memmove (mbc, mbc + mblength, MB_CUR_MAX - mblength);
-	      mbc_pos -= mblength;
-	    }
-	}
-      return;
-    }
-  putchar (c);
-}
-#endif
-
 /* Skip to page PAGE before printing.
    PAGE may be larger than total number of pages. */
 
@@ -2596,9 +2391,6 @@ print_header (void)
   int lhs_spaces;
   int rhs_spaces;
 
-  if (!use_form_feed)
-    printf ("\n\n");
-
   output_position = 0;
   pad_across_to (chars_per_margin);
   print_white_space ();
@@ -2615,7 +2407,7 @@ print_header (void)
   lhs_spaces = available_width >> 1;
   rhs_spaces = available_width - lhs_spaces;
 
-  printf ("%s%*s%s%*s%s\n\n\n",
+  printf ("\n\n%s%*s%s%*s%s\n\n\n",
 	  date_text, lhs_spaces, " ", file_text, rhs_spaces, " ", page_text);
 
   print_a_header = false;
@@ -2717,9 +2509,9 @@ read_line (COLUMN *p)
 	  align_empty_cols = false;
 	}
 
-      if (padding_not_printed - col_sep_width > 0)
+      if (padding_not_printed - col_sep_length > 0)
 	{
-	  pad_across_to (padding_not_printed - col_sep_width);
+	  pad_across_to (padding_not_printed - col_sep_length);
 	  padding_not_printed = ANYWHERE;
 	}
 
@@ -2820,9 +2612,9 @@ print_stored (COLUMN *p)
 	}
     }
 
-  if (padding_not_printed - col_sep_width > 0)
+  if (padding_not_printed - col_sep_length > 0)
     {
-      pad_across_to (padding_not_printed - col_sep_width);
+      pad_across_to (padding_not_printed - col_sep_length);
       padding_not_printed = ANYWHERE;
     }
 
@@ -2835,8 +2627,8 @@ print_stored (COLUMN *p)
   if (spaces_not_printed == 0)
     {
       output_position = p->start_position + end_vector[line];
-      if (p->start_position - col_sep_width == chars_per_margin)
-	output_position -= col_sep_width;
+      if (p->start_position - col_sep_length == chars_per_margin)
+	output_position -= col_sep_length;
     }
 
   return true;
@@ -2855,7 +2647,7 @@ print_stored (COLUMN *p)
    number of characters is 1.) */
 
 static int
-char_to_clump_single (char c)
+char_to_clump (char c)
 {
   unsigned char uc = c;
   char *s = clump_buff;
@@ -2865,10 +2657,10 @@ char_to_clump_single (char c)
   int chars;
   int chars_per_c = 8;
 
-  if (c == input_tab_char[0])
+  if (c == input_tab_char)
     chars_per_c = chars_per_input_tab;
 
-  if (c == input_tab_char[0] || c == '\t')
+  if (c == input_tab_char || c == '\t')
     {
       width = TAB_WIDTH (chars_per_c, input_position);
 
@@ -2939,154 +2731,6 @@ char_to_clump_single (char c)
   return chars;
 }
 
-#ifdef HAVE_MBRTOWC
-static int
-char_to_clump_multi (char c)
-{
-  static size_t mbc_pos = 0;
-  static char mbc[MB_LEN_MAX] = {'\0'};
-  static mbstate_t state = {'\0'};
-  mbstate_t state_bak;
-  wchar_t wc;
-  size_t mblength;
-  int wc_width;
-  register char *s = clump_buff;
-  register int i, j;
-  char esc_buff[4];
-  int width;
-  int chars;
-  int chars_per_c = 8;
-
-  state_bak = state;
-  mbc[mbc_pos++] = c;
-  mblength = mbrtowc (&wc, mbc, mbc_pos, &state);
-
-  width = 0;
-  chars = 0;
-  while (mbc_pos > 0)
-    {
-      switch (mblength)
-	{
-	case (size_t)-2:
-	  state = state_bak;
-	  return 0;
-
-	case (size_t)-1:
-	  state = state_bak;
-	  mblength = 1;
-
-	  if (use_esc_sequence || use_cntrl_prefix)
-	    {
-	      width = +4;
-	      chars = +4;
-	      *s++ = '\\';
-	      sprintf (esc_buff, "%03o", mbc[0]);
-	      for (i = 0; i <= 2; ++i)
-		*s++ = (int) esc_buff[i];
-	    }
-	  else
-	    {
-	      width += 1;
-	      chars += 1;
-	      *s++ = mbc[0];
-	    }
-	  break;
-
-	case 0:
-	  mblength = 1;
-		/* Fall through */
-
-	default:
-	  if (memcmp (mbc, input_tab_char, mblength) == 0)
-	    chars_per_c = chars_per_input_tab;
-
-	  if (memcmp (mbc, input_tab_char, mblength) == 0 || c == '\t')
-	    {
-	      int  width_inc;
-
-	      width_inc = TAB_WIDTH (chars_per_c, input_position);
-	      width += width_inc;
-
-	      if (untabify_input)
-		{
-		  for (i = width_inc; i; --i)
-		    *s++ = ' ';
-		  chars += width_inc;
-		}
-	      else
-		{
-		  for (i = 0; i <  mblength; i++)
-		    *s++ = mbc[i];
-		  chars += mblength;
-		}
-	    }
-	  else if ((wc_width = wcwidth (wc)) < 1)
-	    {
-	      if (use_esc_sequence)
-		{
-		  for (i = 0; i < mblength; i++)
-		    {
-		      width += 4;
-		      chars += 4;
-		      *s++ = '\\';
-		      sprintf (esc_buff, "%03o", c);
-		      for (j = 0; j <= 2; ++j)
-			*s++ = (int) esc_buff[j];
-		    }
-		}
-	      else if (use_cntrl_prefix)
-		{
-		  if (wc < 0200)
-		    {
-		      width += 2;
-		      chars += 2;
-		      *s++ = '^';
-		      *s++ = wc ^ 0100;
-		    }
-		  else
-		    {
-		      for (i = 0; i < mblength; i++)
-			{
-			  width += 4;
-			  chars += 4;
-			  *s++ = '\\';
-			  sprintf (esc_buff, "%03o", c);
-			  for (j = 0; j <= 2; ++j)
-			    *s++ = (int) esc_buff[j];
-			}
-		    }
-		}
-	      else if (wc == L'\b')
-		{
-		  width += -1;
-		  chars += 1;
-		  *s++ = c;
-		}
-	      else
-		{
-		  width += 0;
-		  chars += mblength;
-		  for (i = 0; i < mblength; i++)
-		    *s++ = mbc[i];
-		}
-	    }
-	  else
-	    {
-	      width += wc_width;
-	      chars += mblength;
-	      for (i = 0; i < mblength; i++)
-		*s++ = mbc[i];
-	    }
-	}
-      memmove (mbc, mbc + mblength, MB_CUR_MAX - mblength);
-      mbc_pos -= mblength;
-    }
-
-  input_position += width;
-  return chars;
-}
-#endif
-
 /* We've just printed some files and need to clean up things before
    looking for more options and printing the next batch of files.
 
@@ -3218,10 +2862,10 @@ Mandatory arguments to long options are mandatory for short options too.\n\
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
       fputs (_("\
 \n\
--T implied by -l nn when nn <= 10 or <= 3 with -F. With no FILE, or when\n\
+-t is implied if PAGE_LENGTH <= 10.  With no FILE, or when\n\
 FILE is -, read standard input.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
