@@ -1,10 +1,10 @@
 /* sort - sort lines of text (with all kinds of options).
    Copyright (C) 1988, 1991-2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
    Written December 1988 by Mike Haertel.
    The author may be reached (Email) at the address mike@gnu.ai.mit.edu,
@@ -21,21 +20,12 @@
 
    Ørn E. Hansen added NLS support in 1997.  */
 
-#include <sys/types.h>
-#include <sys/wait.h>
 #include <config.h>
 
-#include <assert.h>
 #include <getopt.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <signal.h>
-#if HAVE_WCHAR_H
-# include <wchar.h>
-#endif
-/* Get isw* functions. */
-#if HAVE_WCTYPE_H
-# include <wctype.h>
-#endif
-
 #include "system.h"
 #include "argmatch.h"
 #include "error.h"
@@ -125,38 +115,14 @@ static int decimal_point;
 /* Thousands separator; if -1, then there isn't one.  */
 static int thousands_sep;
 
-static int force_general_numcompare = 0;
-
 /* Nonzero if the corresponding locales are hard.  */
 static bool hard_LC_COLLATE;
-#if HAVE_LANGINFO_CODESET
+#if HAVE_NL_LANGINFO
 static bool hard_LC_TIME;
 #endif
 
 #define NONZERO(x) ((x) != 0)
 
-/* get a multibyte character's byte length. */
-#define GET_BYTELEN_OF_CHAR(LIM, PTR, MBLENGTH, STATE)			\
-  do									\
-    {									\
-      wchar_t wc;							\
-      mbstate_t state_bak;						\
-									\
-      state_bak = STATE;						\
-      mblength = mbrtowc (&wc, PTR, LIM - PTR, &STATE);			\
-									\
-      switch (MBLENGTH)							\
-	{								\
-	case (size_t)-1:						\
-	case (size_t)-2:						\
-	  STATE = state_bak;						\
-		/* Fall through. */					\
-	case 0:								\
-	  MBLENGTH = 1;							\
-      }									\
-    }									\
-  while (0)
-
 /* The kind of blanks for '-b' to skip in various options. */
 enum blanktype { bl_start, bl_end, bl_both };
 
@@ -294,11 +260,13 @@ static bool reverse;
    they were read if all keys compare equal.  */
 static bool stable;
 
-/* Tab character separating fields.  If tab_length is 0, then fields are
+/* If TAB has this value, blanks separate fields.  */
+enum { TAB_DEFAULT = CHAR_MAX + 1 };
+
+/* Tab character separating fields.  If TAB_DEFAULT, then fields are
    separated by the empty string between a non-blank character and a blank
    character. */
-static char tab[MB_LEN_MAX + 1];
-static size_t tab_length = 0;
+static int tab = TAB_DEFAULT;
 
 /* Flag to remove consecutive duplicate lines from the output.
    Only the last of a sequence of equal lines will be output. */
@@ -411,7 +379,7 @@ The locale specified by the environment affects sort order.\n\
 Set LC_ALL=C to get the traditional sort order that uses\n\
 native byte values.\n\
 "), stdout );
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
 
   exit (status);
@@ -670,44 +638,6 @@ reap_some (void)
     update_proc (pid);
 }
 
-/* Function pointers. */
-static void
-(*inittables) (void);
-static char *
-(*begfield) (const struct line*, const struct keyfield *);
-static char *
-(*limfield) (const struct line*, const struct keyfield *);
-static int
-(*getmonth) (char const *, size_t);
-static int
-(*keycompare) (const struct line *, const struct line *);
-static int
-(*numcompare) (const char *, const char *);
-
-/* Test for white space multibyte character.
-   Set LENGTH the byte length of investigated multibyte character. */
-#if HAVE_MBRTOWC
-static int
-ismbblank (const char *str, size_t len, size_t *length)
-{
-  size_t mblength;
-  wchar_t wc;
-  mbstate_t state;
-
-  memset (&state, '\0', sizeof(mbstate_t));
-  mblength = mbrtowc (&wc, str, len, &state);
-
-  if (mblength == (size_t)-1 || mblength == (size_t)-2)
-    {
-      *length = 1;
-      return 0;
-    }
-
-  *length = (mblength < 1) ? 1 : mblength;
-  return iswblank (wc);
-}
-#endif
-
 /* Clean up any remaining temporary files.  */
 
 static void
@@ -1047,7 +977,7 @@ zaptemp (const char *name)
   free (node);
 }
 
-#if HAVE_LANGINFO_CODESET
+#if HAVE_NL_LANGINFO
 
 static int
 struct_month_cmp (const void *m1, const void *m2)
@@ -1062,7 +992,7 @@ struct_month_cmp (const void *m1, const void *m2)
 /* Initialize the character class tables. */
 
 static void
-inittables_uni (void)
+inittables (void)
 {
   size_t i;
 
@@ -1074,7 +1004,7 @@ inittables_uni (void)
       fold_toupper[i] = toupper (i);
     }
 
-#if HAVE_LANGINFO_CODESET
+#if HAVE_NL_LANGINFO
   /* If we're not in the "C" locale, read different names for months.  */
   if (hard_LC_TIME)
     {
@@ -1100,67 +1030,9 @@ inittables_uni (void)
 #endif
 }
 
-#if HAVE_MBRTOWC
-static void
-inittables_mb (void)
-{
-  int i, j, k, l;
-  char *name, *s;
-  size_t s_len, mblength;
-  char mbc[MB_LEN_MAX];
-  wchar_t wc, pwc;
-  mbstate_t state_mb, state_wc;
-
-  for (i = 0; i < MONTHS_PER_YEAR; i++)
-    {
-      s = (char *) nl_langinfo (ABMON_1 + i);
-      s_len = strlen (s);
-      monthtab[i].name = name = (char *) xmalloc (s_len + 1);
-      monthtab[i].val = i + 1;
-
-      memset (&state_mb, '\0', sizeof (mbstate_t));
-      memset (&state_wc, '\0', sizeof (mbstate_t));
-
-      for (j = 0; j < s_len;)
-	{
-	  if (!ismbblank (s + j, s_len - j, &mblength))
-	    break;
-	  j += mblength;
-	}
-
-      for (k = 0; j < s_len;)
-	{
-	  mblength = mbrtowc (&wc, (s + j), (s_len - j), &state_mb);
-	  assert (mblength != (size_t)-1 && mblength != (size_t)-2);
-	  if (mblength == 0)
-	    break;
-
-	  pwc = towupper (wc);
-	  if (pwc == wc)
-	    {
-	      memcpy (mbc, s + j, mblength);
-	      j += mblength;
-	    }
-	  else
-	    {
-	      j += mblength;
-	      mblength = wcrtomb (mbc, pwc, &state_wc);
-	      assert (mblength != (size_t)0 && mblength != (size_t)-1);
-	    }
-
-	  for (l = 0; l < mblength; l++)
-	    name[k++] = mbc[l];
-	}
-      name[k] = '\0';
-    }
-  qsort ((void *) monthtab, MONTHS_PER_YEAR,
-      sizeof (struct month), struct_month_cmp);
-}
-#endif
-
 /* Specify the amount of main memory to use when sorting.  */
 static void
-specify_sort_size (char const *s)
+specify_sort_size (int oi, char c, char const *s)
 {
   uintmax_t n;
   char *suffix;
@@ -1216,7 +1088,7 @@ specify_sort_size (char const *s)
       e = LONGINT_OVERFLOW;
     }
 
-  STRTOL_FATAL_ERROR (s, _("sort size"), e);
+  xstrtol_fatal (e, oi, c, long_options, s);
 }
 
 /* Return the default sort size.  */
@@ -1368,7 +1240,7 @@ buffer_linelim (struct buffer const *buf)
    by KEY in LINE. */
 
 static char *
-begfield_uni (const struct line *line, const struct keyfield *key)
+begfield (const struct line *line, const struct keyfield *key)
 {
   char *ptr = line->text, *lim = ptr + line->length - 1;
   size_t sword = key->sword;
@@ -1378,10 +1250,10 @@ begfield_uni (const struct line *line, const struct keyfield *key)
   /* The leading field separator itself is included in a field when -t
      is absent.  */
 
-  if (tab_length)
+  if (tab != TAB_DEFAULT)
     while (ptr < lim && sword--)
       {
-	while (ptr < lim && *ptr != tab[0])
+	while (ptr < lim && *ptr != tab)
 	  ++ptr;
 	if (ptr < lim)
 	  ++ptr;
@@ -1409,70 +1281,11 @@ begfield_uni (const struct line *line, const struct keyfield *key)
   return ptr;
 }
 
-#if HAVE_MBRTOWC
-static char *
-begfield_mb (const struct line *line, const struct keyfield *key)
-{
-  int i;
-  char *ptr = line->text, *lim = ptr + line->length - 1;
-  size_t sword = key->sword;
-  size_t schar = key->schar;
-  size_t mblength;
-  mbstate_t state;
-
-  memset (&state, '\0', sizeof(mbstate_t));
-
-  if (tab_length)
-    while (ptr < lim && sword--)
-      {
-	while (ptr < lim && memcmp (ptr, tab, tab_length) != 0)
-	  {
-	    GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	    ptr += mblength;
-	  }
-	if (ptr < lim)
-	  {
-	    GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	    ptr += mblength;
-	  }
-      }
-  else
-    while (ptr < lim && sword--)
-      {
-	while (ptr < lim && ismbblank (ptr, lim - ptr, &mblength))
-	  ptr += mblength;
-	if (ptr < lim)
-	  {
-	    GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	    ptr += mblength;
-	  }
-	while (ptr < lim && !ismbblank (ptr, lim - ptr, &mblength))
-	  ptr += mblength;
-      }
-
-  if (key->skipsblanks)
-    while (ptr < lim && ismbblank (ptr, lim - ptr, &mblength))
-      ptr += mblength;
-
-  for (i = 0; i < schar; i++)
-    {
-      GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-
-      if (ptr + mblength > lim)
-	break;
-      else
-	ptr += mblength;
-    }
-
-  return ptr;
-}
-#endif
-
 /* Return the limit of (a pointer to the first character after) the field
    in LINE specified by KEY. */
 
 static char *
-limfield_uni (const struct line *line, const struct keyfield *key)
+limfield (const struct line *line, const struct keyfield *key)
 {
   char *ptr = line->text, *lim = ptr + line->length - 1;
   size_t eword = key->eword, echar = key->echar;
@@ -1485,10 +1298,10 @@ limfield_uni (const struct line *line, const struct keyfield *key)
      `beginning' is the first character following the delimiting TAB.
      Otherwise, leave PTR pointing at the first `blank' character after
      the preceding field.  */
-  if (tab_length)
+  if (tab != TAB_DEFAULT)
     while (ptr < lim && eword--)
       {
-	while (ptr < lim && *ptr != tab[0])
+	while (ptr < lim && *ptr != tab)
 	  ++ptr;
 	if (ptr < lim && (eword | echar))
 	  ++ptr;
@@ -1534,10 +1347,10 @@ limfield_uni (const struct line *line, const struct keyfield *key)
      */
 
   /* Make LIM point to the end of (one byte past) the current field.  */
-  if (tab_length)
+  if (tab != TAB_DEFAULT)
     {
       char *newlim;
-      newlim = memchr (ptr, tab[0], lim - ptr);
+      newlim = memchr (ptr, tab, lim - ptr);
       if (newlim)
 	lim = newlim;
     }
@@ -1570,107 +1383,6 @@ limfield_uni (const struct line *line, const struct keyfield *key)
   return ptr;
 }
 
-#if HAVE_MBRTOWC
-static char *
-limfield_mb (const struct line *line, const struct keyfield *key)
-{
-  char *ptr = line->text, *lim = ptr + line->length - 1;
-  size_t eword = key->eword, echar = key->echar;
-  int i;
-  size_t mblength;
-  mbstate_t state;
-
-  memset (&state, '\0', sizeof(mbstate_t));
-
-  if (tab_length)
-    while (ptr < lim && eword--)
-      {
-	while (ptr < lim && memcmp (ptr, tab, tab_length) != 0)
-	  {
-	    GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	    ptr += mblength;
-	  }
-	if (ptr < lim && (eword | echar))
-	  {
-	    GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	    ptr += mblength;
-	  }
-      }
-  else
-    while (ptr < lim && eword--)
-      {
-	while (ptr < lim && ismbblank (ptr, lim - ptr, &mblength))
-	  ptr += mblength;
-	if (ptr < lim)
-	  {
-	    GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	    ptr += mblength;
-	  }
-	while (ptr < lim && !ismbblank (ptr, lim - ptr, &mblength))
-	  ptr += mblength;
-      }
-
-
-# ifdef POSIX_UNSPECIFIED
-  /* Make LIM point to the end of (one byte past) the current field.  */
-  if (tab_length)
-    {
-      char *newlim, *p;
-
-      newlim = NULL;
-      for (p = ptr; p < lim;)
- 	{
-	  if (memcmp (p, tab, tab_length) == 0)
-	    {
-	      newlim = p;
-	      break;
-	    }
-
-	  GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	  p += mblength;
-	}
-    }
-  else
-    {
-      char *newlim;
-      newlim = ptr;
-
-      while (newlim < lim && ismbblank (newlim, lim - newlim, &mblength))
-	newlim += mblength;
-      if (ptr < lim)
-	{
-	  GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-	  ptr += mblength;
-	}
-      while (newlim < lim && !ismbblank (newlim, lim - newlim, &mblength))
-	newlim += mblength;
-      lim = newlim;
-    }
-# endif
-
-  /* If we're skipping leading blanks, don't start counting characters
-   *      until after skipping past any leading blanks.  */
-  if (key->skipsblanks)
-    while (ptr < lim && ismbblank (ptr, lim - ptr, &mblength))
-      ptr += mblength;
-
-  memset (&state, '\0', sizeof(mbstate_t));
-
-  /* Advance PTR by ECHAR (if possible), but no further than LIM.  */
-  for (i = 0; i < echar; i++)
-    {
-      GET_BYTELEN_OF_CHAR (lim, ptr, mblength, state);
-
-      if (ptr + mblength > lim)
-	break;
-      else
-	ptr += mblength;
-    }
-
-  return ptr;
-}
-#endif
-
 /* Fill BUF reading from FP, moving buf->left bytes from the end
    of buf->buf to the beginning first.  If EOF is reached and the
    file wasn't terminated by a newline, supply one.  Set up BUF's line
@@ -1753,24 +1465,8 @@ fillbuf (struct buffer *buf, FILE *fp, char const *file)
 		  else
 		    {
 		      if (key->skipsblanks)
-			{
-#if HAVE_MBRTOWC
-			  if (MB_CUR_MAX > 1)
-			    {
-			      size_t mblength;
-			      mbstate_t state;
-			      memset (&state, '\0', sizeof(mbstate_t));
-			      while (line_start < line->keylim &&
-				     ismbblank (line_start,
-						line->keylim - line_start,
-						&mblength))
-				line_start += mblength;
-			    }
-			  else
-#endif
-			  while (blanks[to_uchar (*line_start)])
-			    line_start++;
-			}
+			while (blanks[to_uchar (*line_start)])
+			  line_start++;
 		      line->keybeg = line_start;
 		    }
 		}
@@ -1792,9 +1488,14 @@ fillbuf (struct buffer *buf, FILE *fp, char const *file)
 	  return true;
 	}
 
-      /* The current input line is too long to fit in the buffer.
-	 Double the buffer size and try again.  */
-      buf->buf = X2REALLOC (buf->buf, &buf->alloc);
+      {
+	/* The current input line is too long to fit in the buffer.
+	   Double the buffer size and try again, keeping it properly
+	   aligned.  */
+	size_t line_alloc = buf->alloc / sizeof (struct line);
+	buf->buf = x2nrealloc (buf->buf, &line_alloc, sizeof (struct line));
+	buf->alloc = line_alloc * sizeof (struct line);
+      }
     }
 }
 
@@ -1803,7 +1504,7 @@ fillbuf (struct buffer *buf, FILE *fp, char const *file)
    hideously fast. */
 
 static int
-numcompare_uni (const char *a, const char *b)
+numcompare (const char *a, const char *b)
 {
   while (blanks[to_uchar (*a)])
     a++;
@@ -1813,25 +1514,6 @@ numcompare_uni (const char *a, const char *b)
   return strnumcmp (a, b, decimal_point, thousands_sep);
 }
 
-#if HAVE_MBRTOWC
-static int
-numcompare_mb (const char *a, const char *b)
-{
-  size_t mblength, len;
-  len = strlen (a); /* okay for UTF-8 */
-  while (*a && ismbblank (a, len > MB_CUR_MAX ? MB_CUR_MAX : len, &mblength))
-    {
-      a += mblength;
-      len -= mblength;
-    }
-  len = strlen (b); /* okay for UTF-8 */
-  while (*b && ismbblank (b, len > MB_CUR_MAX ? MB_CUR_MAX : len, &mblength))
-    b += mblength;
-
-  return strnumcmp (a, b, decimal_point, thousands_sep);
-}
-#endif /* HAV_EMBRTOWC */
-
 static int
 general_numcompare (const char *sa, const char *sb)
 {
@@ -1865,7 +1547,7 @@ general_numcompare (const char *sa, const char *sb)
    Return 0 if the name in S is not recognized.  */
 
 static int
-getmonth_uni (char const *month, size_t len)
+getmonth (char const *month, size_t len)
 {
   size_t lo = 0;
   size_t hi = MONTHS_PER_YEAR;
@@ -2020,79 +1702,11 @@ compare_random (char *restrict texta, size_t lena,
   return diff;
 }
 
-#if HAVE_MBRTOWC
-static int
-getmonth_mb (const char *s, size_t len)
-{
-  char *month;
-  register size_t i;
-  register int lo = 0, hi = MONTHS_PER_YEAR, result;
-  char *tmp;
-  size_t wclength, mblength;
-  const char **pp;
-  const wchar_t **wpp;
-  wchar_t *month_wcs;
-  mbstate_t state;
-
-  while (len > 0 && ismbblank (s, len, &mblength))
-    {
-      s += mblength;
-      len -= mblength;
-    }
-
-  if (len == 0)
-    return 0;
-
-  month = (char *) alloca (len + 1);
-
-  tmp = (char *) alloca (len + 1);
-  memcpy (tmp, s, len);
-  tmp[len] = '\0';
-  pp = (const char **)&tmp;
-  month_wcs = (wchar_t *) alloca ((len + 1) * sizeof (wchar_t));
-  memset (&state, '\0', sizeof(mbstate_t));
-
-  wclength = mbsrtowcs (month_wcs, pp, len + 1, &state);
-  assert (wclength != (size_t)-1 && *pp == NULL);
-
-  for (i = 0; i < wclength; i++)
-    {
-      month_wcs[i] = towupper(month_wcs[i]);
-      if (iswblank (month_wcs[i]))
-	{
-	  month_wcs[i] = L'\0';
-	  break;
-	}
-    }
-
-  wpp = (const wchar_t **)&month_wcs;
-
-  mblength = wcsrtombs (month, wpp, len + 1, &state);
-  assert (mblength != (-1) && *wpp == NULL);
-
-  do
-    {
-      int ix = (lo + hi) / 2;
-
-      if (strncmp (month, monthtab[ix].name, strlen (monthtab[ix].name)) < 0)
-	hi = ix;
-      else
-	lo = ix;
-    }
-  while (hi - lo > 1);
-
-  result = (!strncmp (month, monthtab[lo].name, strlen (monthtab[lo].name))
-      ? monthtab[lo].val : 0);
-
-  return result;
-}
-#endif
-
 /* Compare two lines A and B trying every key in sequence until there
    are no more keys or a difference is found. */
 
 static int
-keycompare_uni (const struct line *a, const struct line *b)
+keycompare (const struct line *a, const struct line *b)
 {
   struct keyfield const *key = keylist;
 
@@ -2265,179 +1879,6 @@ keycompare_uni (const struct line *a, const struct line *b)
   return key->reverse ? -diff : diff;
 }
 
-#if HAVE_MBRTOWC
-static int
-keycompare_mb (const struct line *a, const struct line *b)
-{
-  struct keyfield *key = keylist;
-
-  /* For the first iteration only, the key positions have been
-     precomputed for us. */
-  char *texta = a->keybeg;
-  char *textb = b->keybeg;
-  char *lima = a->keylim;
-  char *limb = b->keylim;
-
-  size_t mblength_a, mblength_b;
-  wchar_t wc_a, wc_b;
-  mbstate_t state_a, state_b;
-
-  int diff;
-
-  memset (&state_a, '\0', sizeof(mbstate_t));
-  memset (&state_b, '\0', sizeof(mbstate_t));
-
-  for (;;)
-    {
-      unsigned char *translate = (unsigned char *) key->translate;
-      bool const *ignore = key->ignore;
-
-      /* Find the lengths. */
-      size_t lena = lima <= texta ? 0 : lima - texta;
-      size_t lenb = limb <= textb ? 0 : limb - textb;
-
-      /* Actually compare the fields. */
-      if (key->random)
-        diff = compare_random (texta, lena, textb, lenb);
-      else if (key->numeric | key->general_numeric)
-	{
-	  char savea = *lima, saveb = *limb;
-
-	  *lima = *limb = '\0';
-	  if (force_general_numcompare)
-	    diff = general_numcompare (texta, textb);
-	  else
-	    diff = ((key->numeric ? numcompare : general_numcompare)
-		(texta, textb));
-	  *lima = savea, *limb = saveb;
-	}
-      else if (key->month)
-	diff = getmonth (texta, lena) - getmonth (textb, lenb);
-      else
-	{
-	  if (ignore || translate)
-	    {
-	      char *copy_a = (char *) alloca (lena + 1 + lenb + 1);
-	      char *copy_b = copy_a + lena + 1;
-	      size_t new_len_a, new_len_b;
-	      size_t i, j;
-
-	      /* Ignore and/or translate chars before comparing.  */
-# define IGNORE_CHARS(NEW_LEN, LEN, TEXT, COPY, WC, MBLENGTH, STATE)	\
-  do									\
-    {									\
-      wchar_t uwc;							\
-      char mbc[MB_LEN_MAX];						\
-      mbstate_t state_wc;						\
-									\
-      for (NEW_LEN = i = 0; i < LEN;)					\
-	{								\
-	  mbstate_t state_bak;						\
-									\
-	  state_bak = STATE;						\
-	  MBLENGTH = mbrtowc (&WC, TEXT + i, LEN - i, &STATE);		\
-									\
-	  if (MBLENGTH == (size_t)-2 || MBLENGTH == (size_t)-1		\
-	      || MBLENGTH == 0)						\
-	    {								\
-	      if (MBLENGTH == (size_t)-2 || MBLENGTH == (size_t)-1)	\
-		STATE = state_bak;					\
-	      if (!ignore)						\
-		COPY[NEW_LEN++] = TEXT[i++];				\
-	      continue;							\
-	    }								\
-									\
-	  if (ignore)							\
-	    {								\
-	      if ((ignore == nonprinting && !iswprint (WC))		\
-		   || (ignore == nondictionary				\
-		       && !iswalnum (WC) && !iswblank (WC)))		\
-		{							\
-		  i += MBLENGTH;					\
-		  continue;						\
-		}							\
-	    }								\
-									\
-	  if (translate)						\
-	    {								\
-									\
-	      uwc = towupper(WC);					\
-	      if (WC == uwc)						\
-		{							\
-		  memcpy (mbc, TEXT + i, MBLENGTH);			\
-		  i += MBLENGTH;					\
-		}							\
-	      else							\
-		{							\
-		  i += MBLENGTH;					\
-		  WC = uwc;						\
-		  memset (&state_wc, '\0', sizeof (mbstate_t));		\
-									\
-		  MBLENGTH = wcrtomb (mbc, WC, &state_wc);		\
-		  assert (MBLENGTH != (size_t)-1 && MBLENGTH != 0);	\
-		}							\
-									\
-	      for (j = 0; j < MBLENGTH; j++)				\
-		COPY[NEW_LEN++] = mbc[j];				\
-	    }								\
-	  else								\
-	    for (j = 0; j < MBLENGTH; j++)				\
-	      COPY[NEW_LEN++] = TEXT[i++];				\
-	}								\
-      COPY[NEW_LEN] = '\0';						\
-    }									\
-  while (0)
-	      IGNORE_CHARS (new_len_a, lena, texta, copy_a,
-			    wc_a, mblength_a, state_a);
-	      IGNORE_CHARS (new_len_b, lenb, textb, copy_b,
-			    wc_b, mblength_b, state_b);
-	      diff = xmemcoll (copy_a, new_len_a, copy_b, new_len_b);
-	    }
-	  else if (lena == 0)
-	    diff = - NONZERO (lenb);
-	  else if (lenb == 0)
-	    goto greater;
-	  else
-	    diff = xmemcoll (texta, lena, textb, lenb);
-	}
-
-      if (diff)
-	goto not_equal;
-
-      key = key->next;
-      if (! key)
-	break;
-
-      /* Find the beginning and limit of the next field.  */
-      if (key->eword != -1)
-	lima = limfield (a, key), limb = limfield (b, key);
-      else
-	lima = a->text + a->length - 1, limb = b->text + b->length - 1;
-
-      if (key->sword != -1)
-	texta = begfield (a, key), textb = begfield (b, key);
-      else
-	{
-	  texta = a->text, textb = b->text;
-	  if (key->skipsblanks)
-	    {
-	      while (texta < lima && ismbblank (texta, lima - texta, &mblength_a))
-		texta += mblength_a;
-	      while (textb < limb && ismbblank (textb, limb - textb, &mblength_b))
-		textb += mblength_b;
-	    }
-	}
-    }
-
-  return 0;
-
-greater:
-  diff = 1;
-not_equal:
-  return key->reverse ? -diff : diff;
-}
-#endif
-
 /* Compare two lines A and B, returning negative, zero, or positive
    depending on whether A compares less than, equal to, or greater than B. */
 
@@ -3307,7 +2748,7 @@ main (int argc, char **argv)
   initialize_exit_failure (SORT_FAILURE);
 
   hard_LC_COLLATE = hard_locale (LC_COLLATE);
-#if HAVE_LANGINFO_CODESET
+#if HAVE_NL_LANGINFO
   hard_LC_TIME = hard_locale (LC_TIME);
 #endif
 
@@ -3328,27 +2769,6 @@ main (int argc, char **argv)
       thousands_sep = -1;
   }
 
-#if HAVE_MBRTOWC
-  if (MB_CUR_MAX > 1)
-    {
-      inittables = inittables_mb;
-      begfield = begfield_mb;
-      limfield = limfield_mb;
-      getmonth = getmonth_mb;
-      keycompare = keycompare_mb;
-      numcompare = numcompare_mb;
-    }
-  else
-#endif
-    {
-      inittables = inittables_uni;
-      begfield = begfield_uni;
-      limfield = limfield_uni;
-      getmonth = getmonth_uni;
-      keycompare = keycompare_uni;
-      numcompare = numcompare_uni;
-    }
-
   have_read_stdin = false;
   inittables ();
 
@@ -3422,6 +2842,7 @@ main (int argc, char **argv)
 	 pedantic and a file was seen, unless the POSIX version
 	 predates 1003.1-2001 and -c was not seen and the operand is
 	 "-o FILE" or "-oFILE".  */
+      int oi = -1;
 
       if (c == -1
 	  || (posixly_correct && nfiles != 0
@@ -3431,7 +2852,7 @@ main (int argc, char **argv)
 		    && argv[optind][0] == '-' && argv[optind][1] == 'o'
 		    && (argv[optind][2] || optind + 1 != argc)))
 	  || ((c = getopt_long (argc, argv, short_options,
-				long_options, NULL))
+				long_options, &oi))
 	      == -1))
 	{
 	  if (argc <= optind)
@@ -3458,10 +2879,7 @@ main (int argc, char **argv)
 		  if (! (key->sword | key->schar))
 		    key->sword = SIZE_MAX;
 		  if (! s || *set_ordering (s, key, bl_start))
-		    {
-		      free (key);
-		      key = NULL;
-		    }
+		    key = NULL;
 		  else
 		    {
 		      if (minus_pos_usage)
@@ -3514,7 +2932,7 @@ main (int argc, char **argv)
 	  break;
 
 	case COMPRESS_PROGRAM_OPTION:
-	  if (compress_program && strcmp (compress_program, optarg) != 0)
+	  if (compress_program && !STREQ (compress_program, optarg))
 	    error (SORT_FAILURE, 0, _("multiple compress programs specified"));
 	  compress_program = optarg;
 	  break;
@@ -3594,40 +3012,18 @@ main (int argc, char **argv)
 	  break;
 
 	case 'S':
-	  specify_sort_size (optarg);
+	  specify_sort_size (oi, c, optarg);
 	  break;
 
 	case 't':
 	  {
-	    char newtab[MB_LEN_MAX + 1];
-	    size_t newtab_length = 1;
-	    strncpy (newtab, optarg, MB_LEN_MAX);
-	    if (! newtab[0])
+	    char newtab = optarg[0];
+	    if (! newtab)
 	      error (SORT_FAILURE, 0, _("empty tab"));
-#if HAVE_MBRTOWC
-	    if (MB_CUR_MAX > 1)
-	      {
-		wchar_t wc;
-		mbstate_t state;
-		size_t i;
-
-		memset (&state, '\0', sizeof (mbstate_t));
-		newtab_length = mbrtowc (&wc, newtab, strnlen (newtab,
-							       MB_LEN_MAX),
-					 &state);
-		switch (newtab_length)
-		  {
-		  case (size_t) -1:
-		  case (size_t) -2:
-		  case 0:
-		    newtab_length = 1;
-		  }
-	      }
-#endif
-	    if (newtab_length == 1 && optarg[1])
+	    if (optarg[1])
 	      {
 		if (STREQ (optarg, "\\0"))
-		  newtab[0] = '\0';
+		  newtab = '\0';
 		else
 		  {
 		    /* Provoke with `sort -txx'.  Complain about
@@ -3638,12 +3034,9 @@ main (int argc, char **argv)
 			   quote (optarg));
 		  }
 	      }
-	    if (tab_length
-		&& (tab_length != newtab_length
-		    || memcmp (tab, newtab, tab_length) != 0))
+	    if (tab != TAB_DEFAULT && tab != newtab)
 	      error (SORT_FAILURE, 0, _("incompatible tabs"));
-	    memcpy (tab, newtab, newtab_length);
-	    tab_length = newtab_length;
+	    tab = newtab;
 	  }
 	  break;
 
@@ -3772,6 +3165,7 @@ main (int argc, char **argv)
 	sortfiles[i].name = files[i];
 
       merge (sortfiles, 0, nfiles, outfile);
+      IF_LINT (free (sortfiles));
     }
   else
     sort (files, nfiles, outfile);
