@@ -1,3 +1,21 @@
+# Test "tail".
+
+# Copyright (C) 1997, 1998, 2002, 2004, 2005, 2006 Free Software
+# Foundation, Inc.
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 package Test;
 require 5.002;
 use strict;
@@ -58,6 +76,11 @@ my @tv = (
 ['minus-1', '-', '', '', 0],
 ['minus-2', '-', "x\n" . ("y\n" x 10) . 'z', ("y\n" x 9) . 'z', 0],
 
+['c-2', '-c 2', "abcd\n", "d\n", 0],
+['c-2-minus', '-c 2 --', "abcd\n", "d\n", 0],
+['c2', '-c2', "abcd\n", "d\n", 0],
+['c2-minus', '-c2 --', "abcd\n", "d\n", 0],
+
 ['n-1', '-n 10', "x\n" . ("y\n" x 10) . 'z', ("y\n" x 9) . 'z', 0],
 ['n-2', '-n -10', "x\n" . ("y\n" x 10) . 'z', ("y\n" x 9) . 'z', 0],
 ['n-3', '-n +10', "x\n" . ("y\n" x 10) . 'z', "y\ny\nz", 0],
@@ -72,23 +95,28 @@ my @tv = (
 ['n-5b', '-n  0', "y\n" x 5, '', 0],
 
 # With textutils-1.22, this failed.
-['f-1', '-f -n 1', "a\nb\n", "b\n", 0],
-#['f-2', '-f -n 1 -', "a\nb\n", "b\n", 0],
+['f-pipe-1', '-f -n 1', "a\nb\n", "b\n", 0],
 );
 
 sub test_vector
 {
-  # With _POSIX2_VERSION=199209, `tail -c' succeeds, but err-6 expects
-  # a failure, so set _POSIX2_VERSION to ensure it fails.
-  $Test::env{'err-6'} = ['_POSIX2_VERSION=200112'];
-
   my $t;
   foreach $t (@tv)
     {
       my ($test_name, $flags, $in, $exp, $ret) = @$t;
 
-      $test_name =~ /^(obs-plus-|minus-)/
-	and $Test::env{$test_name} = ['_POSIX2_VERSION=199209'];
+      if ($test_name =~ /^(obs-plus-|minus-)/)
+        {
+	  $Test::env{$test_name} = ['_POSIX2_VERSION=199209'];
+        }
+      if ($test_name =~ /^(err-6|c-2)$/)
+        {
+	  $Test::env{$test_name} = ['_POSIX2_VERSION=200112'];
+        }
+      if ($test_name =~ /^f-pipe-/)
+        {
+	  $Test::env{$test_name} = ['POSIXLY_CORRECT=1'];
+        }
 
       # If you run the minus* tests with a FILE arg they'd hang.
       # If you run the err-1 or err-3 tests with a FILE, they'd misinterpret
